/* Copyright (C) 2004 Hiroyuki OYAMA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "encoding.h"


const char *mtb_find_encoding(const char *name)
{
    const mtb_charset_alias_rec *enc;

    for (enc = mtb_accept_encoding; enc->alias != NULL; enc++) {
        if (strcasecmp(name, enc->alias) == 0)
            return enc->name;
    }
    return NULL;
}


int mtb_iconv_close(iconv_t cd)
{
    if (cd == (iconv_t)(-1))
        return 0;

    return iconv_close(cd);
}


iconv_t mtb_iconv_open(pool *p, const char *from, const char *to)
{
    iconv_t cd;

/*
    if (strcmp(from, to) == 0)
        return (iconv_t)(-1);
*/
    cd = iconv_open(to, from);
    if (cd == (iconv_t)(-1)) {
        return cd;
    }

    return cd;
}

const char *mtb_convert_encoding(pool *p, iconv_t cd, const char *str)
{
    const char *s = str;
    char *new_str, *dist;
    size_t inlen, outlen;

    if (cd == (iconv_t)(-1))
        return str;

    inlen = strlen(s);
    outlen = inlen * 4 + 1;
    dist = new_str = ap_pcalloc(p, outlen);
    if (iconv(cd, &s, &inlen, &new_str, &outlen) == (size_t)(-1)) {
         return str;
    }
    *new_str ='\0';

    return (const char *)dist; 
}

