/* Copyright (C) 2004 Hiroyuki OYAMA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "httpd.h"
#include "http_protocol.h"
#include <string.h>

static void plustospace(char *str)
{
    register int x;
    for(x=0;str[x];x++) if(str[x] == '+') str[x] = ' ';
}

static const char *parse_by_body(request_rec *r)
{
    char *body;
    int rc;
    long length;

    if (ap_setup_client_block(r, REQUEST_CHUNKED_ERROR))
        return "";
    if (!ap_should_client_block(r))
        return "";

    length = r->remaining;
    body = ap_pcalloc(r->pool, length + 1);
    rc = ap_get_client_block(r, body, length);

    return (const char *)body;
}


table *mtb_parse_request(request_rec *r)
{
    table *param = ap_make_table(r->pool, 1);
    const char *args, *chunk;
    char *id, *slash, *dot;

    args = r->args ? r->args : "";
    if (r->method_number == M_POST) {
        args = ap_pstrcat(r->pool, parse_by_body(r), "&", args, NULL);
    }

    while (*args && (chunk = ap_getword(r->pool, &args, '&'))) {
        char *key, *value;

        key = ap_getword(r->pool, &chunk, '=');
        value = ap_getword(r->pool, &chunk, '=');
        plustospace(key);
        plustospace(value);
        ap_unescape_url(key);
        ap_unescape_url(value);
        ap_table_set(param, key, value);
    }

    if (r->path_info != NULL && *(r->path_info) == '/') {
        id = ap_pstrdup(r->pool, r->path_info + 1);
        if ((slash = strchr(id, '/')) != NULL)
            *slash = '\0';
        if ((dot = strchr(id, '.')) != NULL)
            *dot = '\0';

        ap_table_set(param, "id", id);
    }

    return param;
}



const char *mtb_strip_html_tag(pool *p, const char *src)
{
    char *dist, *d, *s;
    size_t l;
    int skip = 0;

    if (src == NULL)
        return NULL;

    l = strlen(src) + 1;
    d = dist = ap_pcalloc(p, l);
    for (s = (char *)src; *s != '\0'; s++) {
        if (*s == '<') {
            skip = 1;
            continue;
        }
        else if (*s == '>') {
            skip = 0;
            continue;
        }
        if (skip)
            continue;

        *d = *s;
        d++;
    }
    *d = '\0';

    return dist; 
}

const char *mtb_escape_html(pool *p, const char *src)
{
    char *dist = "";
    unsigned char *d, *s;

    src = ap_escape_html(p, src);
    dist = ap_palloc(p, strlen(src) * 4 + 1);
    s = (unsigned char *)src;
    d = dist;

    for (; *s; ++s) {
        if (*s == '\n') {
            *d++ = '\\';
            *d++ = 'n';
        } 
        else if (*s == '\r') {
        } 
        else if (*s == '\'') {
            *d++ = '\\';
            *d++ = *s;
        } 
        else {
            *d++ = *s;
        }
    }
    *d = '\0';

    return dist;
}
